@@ -1,4 +1,4 @@
-/* $Id: bridge.c 2040 2008-05-20 22:37:40Z aturner $ */
+/* $Id: bridge.c 2134 2009-01-10 08:10:18Z aturner $ */
 
 /*
  * Copyright (c) 2001-2005 Aaron Turner.
@@ -102,74 +102,95 @@ new_node(void)
 
 
 /**
- * main loop for bridging mode or unidir
+ * main loop for bridging in only one direction
+ * optimized to not use poll(), but rather libpcap's builtin pcap_loop()
  */
-void
-do_bridge(tcpedit_t *tcpedit, pcap_t * pcap1, pcap_t * pcap2)
+static void
+do_bridge_unidirectional(tcpbridge_opt_t *options, tcpedit_t *tcpedit)
+{
+    struct live_data_t livedata;
+    int retcode;
+
+    assert(options);
+    assert(tcpedit);
+    
+    livedata.tcpedit = tcpedit;
+    livedata.source = PCAP_INT1;
+    livedata.pcap = options->pcap1;
+    livedata.options = options;
+
+    if ((retcode = pcap_loop(options->pcap1, options->limit_send, 
+            (pcap_handler)live_callback, (u_char *) &livedata)) < 0) {
+        warnx("Error in pcap_loop(): %s", pcap_geterr(options->pcap1));
+    }
+    
+}
+
+/**
+ * main loop for bridging in both directions.  Since we dealing with two handles
+ * we need to poll() on them which isn't the most efficent
+ */
+static void
+do_bridge_bidirectional(tcpbridge_opt_t *options, tcpedit_t *tcpedit)
 {
     struct pollfd polls[2];     /* one for left & right pcap */
-    int pollresult = 0;
-    u_char source1 = PCAP_INT1;
-    u_char source2 = PCAP_INT2;
+    int pollresult, pollcount, timeout;
     struct live_data_t livedata;
-    int pollcount = 1;          /* default to unidir mode */
     
-    assert(pcap1); /* must be set */
-
-    /* define polls */
-    polls[PCAP_INT1].fd = pcap_fileno(pcap1);
-    polls[PCAP_INT1].events = POLLIN | POLLPRI;
-    polls[PCAP_INT1].revents = 0;
-
-    if (! options.unidir) {
-        assert(pcap2);
-        polls[PCAP_INT2].fd = pcap_fileno(pcap2);
-        polls[PCAP_INT2].events = POLLIN | POLLPRI;
-        polls[PCAP_INT2].revents = 0;
-        pollcount = 2;
-    }
-
-    /* register signals */
-    didsig = 0;
-    (void)signal(SIGINT, catcher);
+    assert(options);
+    assert(tcpedit);
 
     livedata.tcpedit = tcpedit;
+    livedata.options = options;
+
 
     /* 
      * loop until ctrl-C or we've sent enough packets
      * note that if -L wasn't specified, limit_send is
      * set to 0 so this will loop infinately
      */
-    while ((options.limit_send == 0) || (options.limit_send != pkts_sent)) {
-        if (didsig) {
-            packet_stats(&begin, &end, bytes_sent, pkts_sent, failed);
-            exit(1);
-        }
+    while ((options->limit_send == 0) || (options->limit_send > pkts_sent)) {
+        if (didsig)
+            break;
 
         dbgx(3, "limit_send: " COUNTER_SPEC " \t pkts_sent: " COUNTER_SPEC, 
-            options.limit_send, pkts_sent);
+            options->limit_send, pkts_sent);
+
+        /* reset the result codes */
+        polls[PCAP_INT1].revents = 0;
+        polls[PCAP_INT1].events = POLLIN;
+        polls[PCAP_INT1].fd = pcap_fileno(options->pcap1);
+        
+        polls[PCAP_INT2].revents = 0;
+        polls[PCAP_INT2].events = POLLIN;
+        polls[PCAP_INT2].fd = pcap_fileno(options->pcap2);
+
+        timeout = options->poll_timeout;
+        pollcount = 2;
 
         /* poll for a packet on the two interfaces */
-        pollresult = poll(polls, pollcount, options.poll_timeout);
+        pollresult = poll(polls, pollcount, timeout);
 
         /* poll has returned, process the result */
         if (pollresult > 0) {
+            dbgx(3, "pollresult: %d", pollresult);
+            
             /* success, got one or more packets */
             if (polls[PCAP_INT1].revents > 0) {
                 dbg(5, "Processing first interface");
-                livedata.source = source1;
-                livedata.pcap = pcap1;
-                pcap_dispatch(pcap1, -1, (pcap_handler) live_callback,
-                              (u_char *) & livedata);
+                livedata.source = PCAP_INT1;
+                livedata.pcap = options->pcap1;
+                pcap_dispatch(options->pcap1, -1, (pcap_handler) live_callback,
+                              (u_char *) &livedata);
             }
 
             /* check the other interface?? */
-            if (! options.unidir && polls[PCAP_INT2].revents > 0) {
+            if (polls[PCAP_INT2].revents > 0) {
                 dbg(5, "Processing second interface");
-                livedata.source = source2;
-                livedata.pcap = pcap2;
-                pcap_dispatch(pcap2, -1, (pcap_handler) live_callback,
-                              (u_char *) & livedata);
+                livedata.source = PCAP_INT2;
+                livedata.pcap = options->pcap2;
+                pcap_dispatch(options->pcap2, -1, (pcap_handler) live_callback,
+                              (u_char *) &livedata);
             }
 
         }
@@ -182,30 +203,71 @@ do_bridge(tcpedit_t *tcpedit, pcap_t * pcap1, pcap_t * pcap2)
             warnx("poll() error: %s", strerror(errno));
         }
 
-        /* reset the result codes */
-        polls[PCAP_INT1].revents = 0;
-        if (! options.unidir)
-            polls[PCAP_INT2].revents = 0;
-
         /* go back to the top of the loop */
     }
 
-} /* do_bridge() */
+} /* do_bridge_bidirectional() */
+
+
+/**
+ * Main entry point to bridging.  Does some initial setup and then calls the 
+ * correct loop (unidirectional or bidirectional)
+ */
+void
+do_bridge(tcpbridge_opt_t *options, tcpedit_t *tcpedit)
+{   
+    /* do we apply a bpf filter? */
+    if (options->bpf.filter != NULL) {
+        /* compile filter */
+        dbgx(2, "Try to compile pcap bpf filter: %s", options->bpf.filter);
+        if (pcap_compile(options->pcap1, &options->bpf.program, options->bpf.filter, options->bpf.optimize, 0) != 0) {
+            errx(-1, "Error compiling BPF filter: %s", pcap_geterr(options->pcap1));
+        }
+        
+        /* apply filter */
+        pcap_setfilter(options->pcap1, &options->bpf.program);
+
+        /* same for other interface if applicable */
+        if (options->unidir == 0) {
+            /* compile filter */
+            dbgx(2, "Try to compile pcap bpf filter: %s", options->bpf.filter);
+            if (pcap_compile(options->pcap2, &options->bpf.program, options->bpf.filter, options->bpf.optimize, 0) != 0) {
+                errx(-1, "Error compiling BPF filter: %s", pcap_geterr(options->pcap2));
+            }
+        
+            /* apply filter */
+            pcap_setfilter(options->pcap2, &options->bpf.program);
+        }
+    }
+
+    /* register signals */
+    didsig = 0;
+    (void)signal(SIGINT, catcher);
+
+
+    if (options->unidir == 1) {
+        do_bridge_unidirectional(options, tcpedit);
+    } else {
+        do_bridge_bidirectional(options, tcpedit);
+    }
+            
+    packet_stats(&begin, &end, bytes_sent, pkts_sent, failed);
+}
 
 
 /**
  * This is the callback we use with pcap_dispatch to process
  * each packet recieved by libpcap on the two interfaces.
+ * Need to return > 0 to denote success
  */
 static int
 live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
               const u_char * nextpkt)
 {
     ipv4_hdr_t *ip_hdr = NULL;
-    sendpacket_t *sp = NULL;
+    pcap_t *send = NULL;
     static u_char *pktdata = NULL;     /* full packet buffer */
-    u_char **packet;
-    int cache_mode;
+    int cache_mode, retcode;
     static unsigned long packetnum = 0;
     struct macsrc_t *node, finder;  /* rb tree nodes */
 #ifdef DEBUG
@@ -214,7 +276,7 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
     u_int16_t l2proto;
 
     packetnum++;
-    dbgx(2, "packet %d caplen %d", packetnum, pkthdr->caplen);
+    dbgx(2, "packet %lu caplen %d", packetnum, pkthdr->caplen);
 
     /* only malloc the first time */
     if (pktdata == NULL) {
@@ -231,8 +293,8 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
 
 #ifdef ENABLE_VERBOSE
     /* decode packet? */
-    if (options.verbose)
-        tcpdump_print(options.tcpdump, pkthdr, nextpkt);
+    if (livedata->options->verbose)
+        tcpdump_print(livedata->options->tcpdump, pkthdr, nextpkt);
 #endif
 
 
@@ -245,18 +307,17 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
 #endif
 
     /* first, is this a packet sent locally?  If so, ignore it */
-    if ((memcmp(sendpacket_get_hwaddr(options.sp1), &finder.key, 
-            ETHER_ADDR_LEN)) == 0) {
-        dbgx(1, "Packet matches the MAC of %s, skipping.", options.intf1);
+    if ((memcmp(livedata->options->intf1_mac, &finder.key, ETHER_ADDR_LEN)) == 0) {
+        dbgx(1, "Packet matches the MAC of %s, skipping.", livedata->options->intf1);
         return (1);
     }
-    else if ((memcmp(sendpacket_get_hwaddr(options.sp2), &finder.key,
-            ETHER_ADDR_LEN)) == 0) {
-        dbgx(1, "Packet matches the MAC of %s, skipping.", options.intf2);
+    else if ((memcmp(livedata->options->intf2_mac, &finder.key, ETHER_ADDR_LEN)) == 0) {
+        dbgx(1, "Packet matches the MAC of %s, skipping.", livedata->options->intf2);
         return (1);
     }
 
     node = RB_FIND(macsrc_tree, &macsrc_root, &finder);
+    
     /* if we can't find the node, build a new one */
     if (node == NULL) {
         dbg(1, "Unable to find MAC in the tree");
@@ -264,10 +325,11 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
         node->source = livedata->source;
         memcpy(&node->key, &finder.key, ETHER_ADDR_LEN);
         RB_INSERT(macsrc_tree, &macsrc_root, node);
-    }                           /* otherwise compare sources */
+    }
+    
+    /* otherwise compare sources */
     else if (node->source != livedata->source) {
-        dbg(1,
-            "Found the MAC and we had a source missmatch... skipping packet");
+        dbg(1, "Found the dest MAC in the tree and it doesn't match this source NIC... skipping packet");
         /*
          * IMPORTANT!!!
          * Never send a packet out the same interface we sourced it on!
@@ -275,21 +337,20 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
         return (1);
     }
 
-
-
     /* what is our cache mode? */
     cache_mode = livedata->source == PCAP_INT1 ? TCPR_DIR_C2S : TCPR_DIR_S2C;
 
     l2proto = tcpedit_l3proto(livedata->tcpedit, BEFORE_PROCESS, pktdata, pkthdr->len);
     dbgx(2, "Packet protocol: %04hx", l2proto);
+    
     /* should we skip this packet based on CIDR match? */
     if (l2proto == ETHERTYPE_IP) {
         dbg(3, "Packet is IP");
         ip_hdr = (ipv4_hdr_t *)tcpedit_l3data(livedata->tcpedit, BEFORE_PROCESS, pktdata, pkthdr->len);
 
         /* look for include or exclude CIDR match */
-        if (options.xX.cidr != NULL) {
-            if (!process_xX_by_cidr(options.xX.mode, options.xX.cidr, ip_hdr)) {
+        if (livedata->options->xX.cidr != NULL) {
+            if (!process_xX_by_cidr(livedata->options->xX.mode, livedata->options->xX.cidr, ip_hdr)) {
                 dbg(2, "Skipping packet due to CIDR match");
                 return (1);
             }
@@ -297,35 +358,42 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
 
     }
 
-    packet = &pktdata;
-    if (tcpedit_packet(livedata->tcpedit, &pkthdr, packet, cache_mode) == -1) {
-        return -1;
+    if ((retcode = tcpedit_packet(livedata->tcpedit, &pkthdr, &pktdata, cache_mode)) < 0) {
+        if (retcode == TCPEDIT_SOFT_ERROR) {
+            return 1;
+        } else { /* TCPEDIT_ERROR */
+            return -1;
+        }
     }
 
     /* 
      * send packets out the OTHER interface
      * and update the dst mac if necessary
      */
-    if (node->source == PCAP_INT1) {
-        dbgx(2, "Packet source was %s... sending out on %s", options.intf1, 
-            options.intf2);
-        sp = options.sp2;
-    }
-    else if (node->source == PCAP_INT2) {
-        dbgx(2, "Packet source was %s... sending out on %s", options.intf2, 
-            options.intf1);
-        sp = options.sp1;
-    } else {
-        errx(1, "wtf?  our node->source != PCAP_INT1 and != PCAP_INT2: %c", 
-             node->source);
+    switch(node->source) {
+        case PCAP_INT1:
+            dbgx(2, "Packet source was %s... sending out on %s", livedata->options->intf1, 
+                livedata->options->intf2);
+            send = livedata->options->pcap2;
+            break;
+
+        case PCAP_INT2:
+            dbgx(2, "Packet source was %s... sending out on %s", livedata->options->intf2, 
+                livedata->options->intf1);
+            send = livedata->options->pcap1;
+            break;
+        
+        default:
+            errx(-1, "wtf?  our node->source != PCAP_INT1 and != PCAP_INT2: %c", 
+                 node->source);        
     }
 
     /*
      * write packet out on the network 
      */
-     if (sendpacket(sp, pktdata, pkthdr->caplen) < (int)pkthdr->caplen) {
-         errx(1, "Unable to send packet out %s: %s", sp->device, sendpacket_geterr(sp));
-     }
+     if (pcap_sendpacket(send, pktdata, pkthdr->caplen) < 0)
+         errx(-1, "Unable to send packet out %s: %s", 
+            send == livedata->options->pcap1 ? livedata->options->intf1 : livedata->options->intf2, pcap_geterr(send));
 
     bytes_sent += pkthdr->caplen;
     pkts_sent++;
@@ -336,11 +404,4 @@ live_callback(struct live_data_t *livedata, struct pcap_pkthdr *pkthdr,
     return (1);
 } /* live_callback() */
 
-/*
- Local Variables:
- mode:c
- indent-tabs-mode:nil
- c-basic-offset:4
- End:
-*/
 
