@@ -3,12 +3,19 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: randutil.c 2000 2008-04-27 06:17:35Z aturner $
+ * $Id: randutil.c 2035 2008-05-19 05:35:24Z aturner $
  */
 
 #include "config.h"
 
+#ifdef HAVE_LIBDNET
+/* need to undef these which are pulled in via defines.h, prior to importing dnet.h */
+#undef icmp_id
+#undef icmp_seq
+#undef icmp_data
+#undef icmp_mask
 #include <dnet.h>
+#endif
 
 #include <err.h>
 #include <stdio.h>
