@@ -1,4 +1,4 @@
-/* $Id: tcpedit.c 2040 2008-05-20 22:37:40Z aturner $ */
+/* $Id: tcpedit.c 2109 2009-01-07 20:15:25Z aturner $ */
 
 /*
  * Copyright (c) 2001-2007 Aaron Turner.
@@ -72,36 +72,21 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
 {
     ipv4_hdr_t *ip_hdr = NULL;
     arp_hdr_t *arp_hdr = NULL;
-    int l2len = 0, l2proto, retval = 0, dlt, pktlen, lendiff;
+    int l2len = 0, l2proto, retval = 0, dst_dlt, src_dlt, pktlen, lendiff;
     int needtorecalc = 0;           /* did the packet change? if so, checksum */
-    static u_char *packet = NULL;   /* static buffer to hold packet data when padding out */
-    
+    u_char *packet = *pktdata;
     assert(tcpedit);
     assert(pkthdr);
     assert(*pkthdr);
     assert(pktdata);
-    assert(*pktdata);
+    assert(packet);
     assert(tcpedit->validated);
 
 
     tcpedit->runtime.packetnum++;
-    dbgx(2, "packet " COUNTER_SPEC " caplen %d", 
+    dbgx(3, "packet " COUNTER_SPEC " caplen %d", 
             tcpedit->runtime.packetnum, (*pkthdr)->caplen);
 
-    /* 
-     * copy over our packet data.  Necessary because many options like padding or 
-     * adding VLAN tags makes the packet size increase and we'll get segfaults 
-     * later on
-     */
-            
-    /* allocate our buffer the first time */
-    if (packet == NULL)
-        packet = safe_malloc(MAX_SNAPLEN);
-
-    /* and copy over the packet data... */
-    memcpy(packet, *pktdata, (*pkthdr)->caplen);
-    *pktdata = packet;
-    
     /*
      * remove the Ethernet FCS (checksum)?
      * note that this feature requires the end user to be smart and
@@ -112,14 +97,23 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         (*pkthdr)->caplen -= 4;
         (*pkthdr)->len -= 4;
     }
-        
-    /* rewrite DLT */
-    if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, *pktdata, (*pkthdr)->caplen, direction)) == TCPEDIT_ERROR)
-        errx(1, "%s", tcpedit_geterr(tcpedit));
 
-    /* unable to edit packet, most likely 802.11 management frame */
+    src_dlt = tcpedit_dlt_src(tcpedit->dlt_ctx);
+    
+    /* not everything has a L3 header, so check for errors.  returns proto in network byte order */
+    if ((l2proto = tcpedit_dlt_proto(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen)) < 0) {
+        dbg(2, "Packet has no L3+ header");
+    } else {
+        dbgx(2, "Layer 3 protocol type is: 0x%04x", ntohs(l2proto));
+    }
+        
+    /* rewrite Layer 2 */
+    if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, pktdata, (*pkthdr)->caplen, direction)) == TCPEDIT_ERROR)
+        errx(-1, "%s", tcpedit_geterr(tcpedit));
+
+    /* unable to edit packet, most likely 802.11 management or data QoS frame */
     if (pktlen == TCPEDIT_SOFT_ERROR) {
-        dbgx(1, "%s", tcpedit_geterr(tcpedit));
+        dbgx(3, "%s", tcpedit_geterr(tcpedit));
         return TCPEDIT_SOFT_ERROR;
     }
 
@@ -127,28 +121,51 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     lendiff = pktlen - (*pkthdr)->caplen;
     (*pkthdr)->caplen += lendiff;
     (*pkthdr)->len += lendiff;
+    
+    dst_dlt = tcpedit_dlt_dst(tcpedit->dlt_ctx);
+    l2len = tcpedit_dlt_l2len(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen);
 
-    dlt = tcpedit_dlt_dst(tcpedit->dlt_ctx);
-    l2proto = tcpedit_dlt_proto(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen);
-    l2len = tcpedit_dlt_l2len(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen);
+    dbgx(2, "dst_dlt = %04x\tsrc_dlt = %04x\tproto = %04x\tl2len = %d", dst_dlt, src_dlt, ntohs(l2proto), l2len);
 
     /* does packet have an IP header?  if so set our pointer to it */
     if (l2proto == htons(ETHERTYPE_IP)) {
-        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen);
+        ip_hdr = (ipv4_hdr_t *)tcpedit_dlt_l3data(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen);
         if (ip_hdr == NULL) {
             return TCPEDIT_ERROR;
         }        
-        dbg(3, "Packet has an IPv4 header...");
+        dbgx(3, "Packet has an IPv4 header: %p...", ip_hdr);
     } else {
-        dbgx(3, "Packet isn't IPv4: 0x%02x", l2proto);
+        dbgx(3, "Packet isn't IPv4: 0x%04x", l2proto);
         /* non-IP packets have a NULL ip_hdr struct */
         ip_hdr = NULL;
     }
 
-    /* rewrite the TTL */
-    needtorecalc += rewrite_ipv4_ttl(tcpedit, ip_hdr);
+    /* The following edits only apply for IPv4 */
+    if (ip_hdr != NULL) {
+        
+        /* set TOS ? */
+        if (tcpedit->tos > -1)
+            ip_hdr->ip_tos = tcpedit->tos;
+            
+        /* rewrite the TTL */
+        needtorecalc += rewrite_ipv4_ttl(tcpedit, ip_hdr);
+
+        /* rewrite TCP/UDP ports */
+        if (tcpedit->portmap != NULL) {
+            if ((retval = rewrite_ports(tcpedit, &ip_hdr)) < 0)
+                return TCPEDIT_ERROR;
+            needtorecalc += retval;
+        }
+
+        /* Untruncate packet? Only works for IP packets */
+        if (tcpedit->fixlen) {
+            if ((retval = untrunc_packet(tcpedit, *pkthdr, packet, ip_hdr)) < 0)
+                return TCPEDIT_ERROR;
+            needtorecalc += retval;
+        }
+    }
     
-    /* rewrite IP addresses */
+    /* rewrite IP addresses in IPv4 or ARP */
     if (tcpedit->rewrite_ip) {
         /* IP packets */
         if (ip_hdr != NULL) {
@@ -159,7 +176,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
 
         /* ARP packets */
         else if (l2proto == htons(ETHERTYPE_ARP)) {
-            arp_hdr = (arp_hdr_t *)(&(*pktdata)[l2len]);
+            arp_hdr = (arp_hdr_t *)&(packet[l2len]);
             /* unlike, rewrite_ipl3, we don't care if the packet changed
              * because we never need to recalc the checksums for an ARP
              * packet.  So ignore the return value
@@ -169,42 +186,31 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         }
     }
 
-    /* rewrite ports */
-    if (tcpedit->portmap != NULL && (ip_hdr != NULL)) {
-        if ((retval = rewrite_ports(tcpedit, &ip_hdr)) < 0)
-            return TCPEDIT_ERROR;
-        needtorecalc += retval;
-    }
 
-    /* Untruncate packet? Only for IP packets */
-    if ((tcpedit->fixlen) && (ip_hdr != NULL)) {
-        if ((retval = untrunc_packet(tcpedit, *pkthdr, *pktdata, ip_hdr)) < 0)
-            return TCPEDIT_ERROR;
-        needtorecalc += retval;
-    }
-
-
-    /* do we need to spoof the src/dst IP address? */
+    /* do we need to spoof the src/dst IP address in IPv4 or ARP? */
     if (tcpedit->seed) {
+        /* IPv4 Packets */
         if (ip_hdr != NULL) {
-            if ((retval = randomize_ipv4(tcpedit, *pkthdr, *pktdata, 
+            if ((retval = randomize_ipv4(tcpedit, *pkthdr, packet, 
                     ip_hdr)) < 0)
                 return TCPEDIT_ERROR;
             needtorecalc += retval;
-        } else {
+
+        /* ARP packets */
+        } else if (l2proto == htons(ETHERTYPE_ARP)) {
             if (direction == TCPR_DIR_C2S) {
-                if (randomize_iparp(tcpedit, *pkthdr, *pktdata, 
+                if (randomize_iparp(tcpedit, *pkthdr, packet, 
                         tcpedit->runtime.dlt1) < 0)
                     return TCPEDIT_ERROR;
             } else {
-                if (randomize_iparp(tcpedit, *pkthdr, *pktdata, 
+                if (randomize_iparp(tcpedit, *pkthdr, packet, 
                         tcpedit->runtime.dlt2) < 0)
                     return TCPEDIT_ERROR;
             }
         }
     }
 
-    /* do we need to fix checksums? */
+    /* do we need to fix checksums? -- must always do this last! */
     if ((tcpedit->fixcsum || needtorecalc) && (ip_hdr != NULL)) {
         retval = fix_checksums(tcpedit, *pkthdr, ip_hdr);
         if (retval < 0) {
@@ -215,7 +221,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     }
 
     
-    tcpedit_dlt_merge_l3data(tcpedit->dlt_ctx, dlt, *pktdata, (*pkthdr)->caplen, (u_char *)ip_hdr);
+    tcpedit_dlt_merge_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen, (u_char *)ip_hdr);
 
     tcpedit->runtime.total_bytes += (*pkthdr)->caplen;
     tcpedit->runtime.pkts_edited ++;
@@ -237,6 +243,7 @@ tcpedit_init(tcpedit_t **tcpedit_ex, int dlt)
         return TCPEDIT_ERROR;
 
     tcpedit->mtu = DEFAULT_MTU; /* assume 802.3 Ethernet */
+    tcpedit->tos = -1; /* disabled by default */
  
     memset(&(tcpedit->runtime), 0, sizeof(tcpedit_runtime_t));
     tcpedit->runtime.dlt1 = dlt;
@@ -345,11 +352,8 @@ tcpedit_setwarn(tcpedit_t *tcpedit, const char *fmt, ...)
     assert(tcpedit);
 
     va_start(ap, fmt);
-    if (fmt != NULL) {
-        dbgx(1, fmt, ap);
-        (void)vsnprintf(tcpedit->runtime.warnstr, 
-              (TCPEDIT_ERRSTR_LEN - 1), fmt, ap);
-    }
+    if (fmt != NULL)
+        (void)vsnprintf(tcpedit->runtime.warnstr, (TCPEDIT_ERRSTR_LEN - 1), fmt, ap);
 
     va_end(ap);
         
